@@ -19,10 +19,12 @@
 
 #include <upolynomial.h>
 #include <feasibility_set.h>
+#include <feasibility_set_int.h>
 #include <variable_db.h>
 #include <variable_list.h>
 
 #include "polynomial/polynomial.h"
+#include "polynomial/coefficient.h"
 
 #include "polynomial/gcd.h"
 #include "polynomial/factorization.h"
@@ -40,7 +42,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#define SWAP(type, x, y) { type tmp = x; x = y; y = tmp; }
+#define SWAP(type, x, y) ({ type tmp = x; x = y; y = tmp; })
 
 static
 void check_polynomial_assignment(const lp_polynomial_t* A, const lp_assignment_t* M, lp_variable_t x) {
@@ -60,7 +62,7 @@ void check_polynomial_assignment(const lp_polynomial_t* A, const lp_assignment_t
     lp_variable_t y = vars.list[i];
     if (x != y && lp_assignment_get_value(M, y)->type == LP_VALUE_NONE) {
       lp_polynomial_print(A, trace_out);
-      tracef("\n")
+      tracef("\n");
       assert(0);
     }
   }
@@ -172,8 +174,6 @@ void lp_polynomial_set_external(lp_polynomial_t* A) {
   }
 }
 
-#define SWAP(type, x, y) { type tmp = x; x = y; y = tmp; }
-
 void lp_polynomial_swap(lp_polynomial_t* A1, lp_polynomial_t* A2) {
   // Swap everything, but keep the external flags
   lp_polynomial_t tmp = *A1; *A1 = *A2; *A2 = tmp;
@@ -207,6 +207,10 @@ int lp_polynomial_lc_sgn(const lp_polynomial_t* A) {
   return coefficient_lc_sgn(A->ctx, &A->data);
 }
 
+void lp_polynomial_lc_constant(const lp_polynomial_t* A, lp_integer_t *out) {
+  lp_polynomial_external_clean(A);
+  coefficient_lc_constant(A->ctx, &A->data, out);
+}
 
 size_t lp_polynomial_degree(const lp_polynomial_t* A) {
   lp_polynomial_external_clean(A);
@@ -258,6 +262,7 @@ int lp_polynomial_is_univariate(const lp_polynomial_t* A) {
   return coefficient_is_univariate(&A->data);
 }
 
+// TODO is a constant polynomial univariate?
 int lp_polynomial_is_univariate_m(const lp_polynomial_t* A, const lp_assignment_t* m) {
   if (lp_polynomial_is_constant(A)) {
     return 0;
@@ -283,13 +288,27 @@ int lp_polynomial_is_univariate_m(const lp_polynomial_t* A, const lp_assignment_
 }
 
 lp_upolynomial_t* lp_polynomial_to_univariate(const lp_polynomial_t* A) {
-  if (!coefficient_is_univariate(&A->data)) {
-    return 0;
+  lp_polynomial_external_clean(A);
+  if (!(coefficient_is_constant(&A->data) || coefficient_is_univariate(&A->data))) {
+    return NULL;
   } else {
     return coefficient_to_univariate(A->ctx, &A->data);
   }
 }
 
+lp_upolynomial_t* lp_polynomial_to_univariate_m(const lp_polynomial_t* A, const lp_assignment_t* m) {
+  lp_polynomial_external_clean(A);
+  return coefficient_to_univariate_m(A->ctx, &A->data, m);
+}
+
+int lp_polynomial_is_monomial(const lp_polynomial_t* A) {
+  return coefficient_is_monomial(A->ctx, &A->data);
+}
+
+void lp_polynomial_to_monomial(const lp_polynomial_t* A, lp_monomial_t* out) {
+  coefficient_to_monomial(A->ctx, &A->data, out);
+}
+
 int lp_polynomial_is_assigned(const lp_polynomial_t* A, const lp_assignment_t* m) {
   lp_polynomial_external_clean(A);
   return coefficient_is_assigned(A->ctx, &A->data, m);
@@ -320,6 +339,16 @@ lp_value_t* lp_polynomial_evaluate(const lp_polynomial_t* A, const lp_assignment
   return coefficient_evaluate(A->ctx, &A->data, m);
 }
 
+void lp_polynomial_evaluate_integer(const lp_polynomial_t* A, const lp_assignment_t* m, lp_integer_t *value) {
+  lp_polynomial_external_clean(A);
+
+  if (trace_is_enabled("polynomial::check_input")) {
+    check_polynomial_assignment(A, m, lp_variable_null);
+  }
+
+  coefficient_evaluate_integer(A->ctx, &A->data, m, value);
+}
+
 int lp_polynomial_cmp(const lp_polynomial_t* A1, const lp_polynomial_t* A2) {
 
   if (trace_is_enabled("polynomial")) {
@@ -626,6 +655,31 @@ void lp_polynomial_prem(lp_polynomial_t* R, const lp_polynomial_t* A1, const lp_
   }
 }
 
+void lp_polynomial_pdivrem(lp_polynomial_t* D, lp_polynomial_t* R, const lp_polynomial_t* A1, const lp_polynomial_t* A2) {
+
+    if (trace_is_enabled("polynomial")) {
+        tracef("polynomial_pdirvrem("); lp_polynomial_print(D, trace_out); tracef(", "); lp_polynomial_print(R, trace_out); tracef(", "); lp_polynomial_print(A1, trace_out); tracef(", "); lp_polynomial_print(A2, trace_out); tracef(")\n");
+        lp_variable_order_print(
+                A1->ctx->var_order, A1->ctx->var_db,
+                trace_out);
+        tracef("\n");
+    }
+
+    assert(lp_polynomial_context_equal(A1->ctx, A2->ctx));
+
+    lp_polynomial_external_clean(A1);
+    lp_polynomial_external_clean(A2);
+
+    lp_polynomial_set_context(D, A1->ctx);
+    lp_polynomial_set_context(R, A1->ctx);
+
+    coefficient_pdivrem(D->ctx, &D->data, &R->data, &A1->data, &A2->data);
+
+    if (trace_is_enabled("polynomial")) {
+        tracef("polynomial_pdirvrem() => ("); lp_polynomial_print(D, trace_out); tracef(", "); lp_polynomial_print(R, trace_out); tracef(")\n");
+    }
+}
+
 void lp_polynomial_sprem(lp_polynomial_t* R, const lp_polynomial_t* A1, const lp_polynomial_t* A2) {
 
   if (trace_is_enabled("polynomial")) {
@@ -650,6 +704,31 @@ void lp_polynomial_sprem(lp_polynomial_t* R, const lp_polynomial_t* A1, const lp
   }
 }
 
+void lp_polynomial_spdivrem(lp_polynomial_t* D, lp_polynomial_t* R, const lp_polynomial_t* A1, const lp_polynomial_t* A2) {
+
+    if (trace_is_enabled("polynomial")) {
+        tracef("lp_polynomial_spdirvrem("); lp_polynomial_print(D, trace_out); tracef(", "); lp_polynomial_print(R, trace_out); tracef(", "); lp_polynomial_print(A1, trace_out); tracef(", "); lp_polynomial_print(A2, trace_out); tracef(")\n");
+        lp_variable_order_print(
+                A1->ctx->var_order, A1->ctx->var_db,
+                trace_out);
+        tracef("\n");
+    }
+
+    assert(lp_polynomial_context_equal(A1->ctx, A2->ctx));
+
+    lp_polynomial_external_clean(A1);
+    lp_polynomial_external_clean(A2);
+
+    lp_polynomial_set_context(D, A1->ctx);
+    lp_polynomial_set_context(R, A1->ctx);
+
+    coefficient_spdivrem(D->ctx, &D->data, &R->data, &A1->data, &A2->data);
+
+    if (trace_is_enabled("polynomial")) {
+        tracef("lp_polynomial_spdirvrem() => ("); lp_polynomial_print(D, trace_out); tracef(", "); lp_polynomial_print(R, trace_out); tracef(")\n");
+    }
+}
+
 void lp_polynomial_divrem(lp_polynomial_t* D, lp_polynomial_t* R, const lp_polynomial_t* A1, const lp_polynomial_t* A2) {
 
   if (trace_is_enabled("polynomial")) {
@@ -668,7 +747,7 @@ void lp_polynomial_divrem(lp_polynomial_t* D, lp_polynomial_t* R, const lp_polyn
   lp_polynomial_set_context(D, A1->ctx);
   lp_polynomial_set_context(R, A1->ctx);
 
-  coefficient_divrem(D->ctx, &R->data, &R->data, &A1->data, &A2->data);
+  coefficient_divrem(D->ctx, &D->data, &R->data, &A1->data, &A2->data);
 
   if (trace_is_enabled("polynomial")) {
     tracef("polynomial_rem() => ("); lp_polynomial_print(D, trace_out); tracef(", "); lp_polynomial_print(R, trace_out); tracef(")\n");
@@ -754,10 +833,10 @@ void lp_polynomial_reduce(
   lp_polynomial_set_context(Q, ctx);
   lp_polynomial_set_context(R, ctx);
 
-  coefficient_reduce(ctx, &A->data, &B->data, &P->data, &Q->data, &R->data, 1);
+  coefficient_reduce(ctx, &A->data, &B->data, &P->data, &Q->data, &R->data, REMAINDERING_PSEUDO_DENSE);
 
   if (trace_is_enabled("polynomial")) {
-    tracef("polynomial_derivative() =>\n");
+    tracef("polynomial_reduce() =>\n");
     tracef("\t P = "); lp_polynomial_print(P, trace_out); tracef("\n");
     tracef("\t Q = "); lp_polynomial_print(Q, trace_out); tracef("\n");
     tracef("\t R = "); lp_polynomial_print(R, trace_out); tracef("\n");
@@ -834,6 +913,7 @@ void lp_polynomial_psc(lp_polynomial_t** psc, const lp_polynomial_t* A, const lp
   coefficient_psc(ctx, psc_coeff, &A->data, &B->data);
 
   // Construct the output (one less, we ignore the final 1)
+  // the last one is not ignored right now
   for (i = 0; i < size; ++ i) {
     lp_polynomial_t tmp;
     lp_polynomial_construct_from_coefficient(&tmp, ctx, psc_coeff + i);
@@ -851,6 +931,72 @@ void lp_polynomial_psc(lp_polynomial_t** psc, const lp_polynomial_t* A, const lp
   }
 }
 
+void lp_polynomial_subres(lp_polynomial_t** subres, const lp_polynomial_t* A, const lp_polynomial_t* B) {
+
+  if (trace_is_enabled("polynomial")) {
+    tracef("polynomial_subres("); lp_polynomial_print(A, trace_out); tracef(", "); lp_polynomial_print(B, trace_out); tracef(")\n");
+  }
+
+  if (trace_is_enabled("polynomial::expensive")) {
+    tracef("A = "); lp_polynomial_print(A, trace_out); tracef("\n");
+    tracef("B = "); lp_polynomial_print(B, trace_out); tracef("\n");
+    tracef("var = %s\n", lp_variable_db_get_name(A->ctx->var_db, lp_polynomial_top_variable(A)));
+    lp_variable_order_print(A->ctx->var_order, A->ctx->var_db, trace_out); tracef("\n");
+  }
+
+  assert(A->data.type == COEFFICIENT_POLYNOMIAL);
+  assert(B->data.type == COEFFICIENT_POLYNOMIAL);
+  assert(VAR(&A->data) == VAR(&B->data));
+
+  size_t A_deg = lp_polynomial_degree(A);
+  size_t B_deg = lp_polynomial_degree(B);
+
+  if (A_deg < B_deg) {
+    lp_polynomial_subres(subres, B, A);
+    return;
+  }
+
+  const lp_polynomial_context_t* ctx = A->ctx;
+  assert(lp_polynomial_context_equal(B->ctx, ctx));
+
+  if (trace_is_enabled("polynomial")) {
+    lp_variable_order_print(A->ctx->var_order, A->ctx->var_db, trace_out);
+    tracef("\n");
+  }
+
+  lp_polynomial_external_clean(A);
+  lp_polynomial_external_clean(B);
+
+  // Allocate the space for the result
+  size_t size = B_deg + 1;
+  coefficient_t *subres_coeff = malloc(sizeof(coefficient_t) * size);
+  size_t i;
+  for (i = 0; i < size; ++ i) {
+    coefficient_construct(ctx, subres_coeff + i);
+  }
+
+  // Compute
+  coefficient_subres(ctx, subres_coeff, &A->data, &B->data);
+
+  // Construct the output (one less, we ignore the final 1)
+  // the last one is not ignored right now
+  for (i = 0; i < size; ++ i) {
+    lp_polynomial_t tmp;
+    lp_polynomial_construct_from_coefficient(&tmp, ctx, subres_coeff + i);
+    lp_polynomial_swap(&tmp, subres[i]);
+    lp_polynomial_destruct(&tmp);
+    coefficient_destruct(&subres_coeff[i]);
+  }
+
+  free(subres_coeff);
+
+  if (trace_is_enabled("polynomial")) {
+    for (i = 0; i < size; ++ i) {
+      tracef("SR[%zu] = ", i); lp_polynomial_print(subres[i], trace_out); tracef("\n");
+    }
+  }
+}
+
 lp_polynomial_vector_t* lp_polynomial_mgcd(const lp_polynomial_t* A, const lp_polynomial_t* B, const lp_assignment_t* m) {
 
   if (trace_is_enabled("polynomial")) {
@@ -1095,7 +1241,7 @@ void lp_polynomial_roots_isolate(const lp_polynomial_t* A, const lp_assignment_t
   }
 
   if (trace_is_enabled("polynomial")) {
-    tracef("polynomial_root_isolate("); lp_polynomial_print(A, trace_out); tracef("): unsorted roots\n")
+    tracef("polynomial_root_isolate("); lp_polynomial_print(A, trace_out); tracef("): unsorted roots\n");
     for (i = 0; i < roots_tmp_size; ++ i) {
       tracef("%zu :", i); lp_value_print(roots_tmp + i, trace_out); tracef("\n");
     }
@@ -1106,7 +1252,7 @@ void lp_polynomial_roots_isolate(const lp_polynomial_t* A, const lp_assignment_t
     qsort(roots_tmp, roots_tmp_size, sizeof(lp_value_t), lp_value_cmp_void);
 
     if (trace_is_enabled("polynomial")) {
-      tracef("polynomial_root_isolate("); lp_polynomial_print(A, trace_out); tracef("): sorted roots\n")
+      tracef("polynomial_root_isolate("); lp_polynomial_print(A, trace_out); tracef("): sorted roots\n");
       for (i = 0; i < roots_tmp_size; ++ i) {
         tracef("%zu :", i); lp_value_print(roots_tmp + i, trace_out); tracef("\n");
       }
@@ -1378,7 +1524,7 @@ int lp_polynomial_constraint_infer_bounds(const lp_polynomial_t* A, lp_sign_cond
     if (r) {
       return r;
     }
-    return lp_polynomial_constraint_infer_bounds(A, LP_SGN_LE_0, 0, M);
+    return lp_polynomial_constraint_infer_bounds(A, LP_SGN_GE_0, 0, M);
   }
   case LP_SGN_NE_0: // |x| - d != 0 => ?
     return 0;
@@ -1597,7 +1743,7 @@ lp_polynomial_t* lp_polynomial_constraint_explain_infer_bounds(const lp_polynomi
     if (p) {
       return p;
     }
-    return lp_polynomial_constraint_explain_infer_bounds(A, LP_SGN_LE_0, 0, x);
+    return lp_polynomial_constraint_explain_infer_bounds(A, LP_SGN_GE_0, 0, x);
   }
   case LP_SGN_NE_0: // |x| - d != 0 => ?
     return 0;
@@ -1891,6 +2037,50 @@ lp_feasibility_set_t* lp_polynomial_root_constraint_get_feasible_set(const lp_po
 
 }
 
+lp_feasibility_set_int_t* lp_polynomial_constraint_get_feasible_set_Zp(const lp_polynomial_t* A, lp_sign_condition_t sgn_condition, int negated, const lp_assignment_t* M) {
+  const lp_polynomial_context_t *ctx = lp_polynomial_get_context(A);
+  assert(ctx->K != lp_Z);
+  assert(lp_sign_condition_Zp_valid(sgn_condition));
+
+  // The ring
+  lp_int_ring_t *K = ctx->K;
+
+  // Negate the constraint if negated
+  if (negated) {
+    sgn_condition = lp_sign_condition_negate(sgn_condition);
+  }
+
+  assert(lp_polynomial_is_univariate_m(A, M));
+
+  lp_upolynomial_t *upoly = lp_polynomial_to_univariate_m(A, M);
+  if (lp_upolynomial_degree(upoly) == 0) {
+    if (lp_upolynomial_is_zero(upoly)) {
+      return sgn_condition == LP_SGN_EQ_0 ? lp_feasibility_set_int_new_full(K) : lp_feasibility_set_int_new_empty(K);
+    } else {
+      return sgn_condition == LP_SGN_EQ_0 ? lp_feasibility_set_int_new_empty(K) : lp_feasibility_set_int_new_full(K);
+    }
+  }
+
+  lp_feasibility_set_int_t *result = sgn_condition == LP_SGN_EQ_0 ? lp_feasibility_set_int_new_empty(K) : lp_feasibility_set_int_new_full(K);
+
+  lp_upolynomial_roots_find_Zp(upoly, &result->elements, &result->size);
+
+#ifndef NDEBUG
+  {
+    lp_integer_t tmp;
+    lp_integer_construct(&tmp);
+    for (size_t i = 0; i < result->size; ++i) {
+      lp_upolynomial_evaluate_at_integer(upoly, &result->elements[i], &tmp);
+      assert(lp_integer_is_zero(lp_upolynomial_ring(upoly) , &tmp));
+    }
+    lp_integer_destruct(&tmp);
+  }
+#endif
+
+  lp_upolynomial_delete(upoly);
+  return result;
+}
+
 void lp_polynomial_get_variables(const lp_polynomial_t* A, lp_variable_list_t* vars) {
   coefficient_get_variables(&A->data, vars);
 }
@@ -1937,12 +2127,33 @@ int lp_polynomial_constraint_evaluate(const lp_polynomial_t* A, lp_sign_conditio
   if (trace_is_enabled("polynomial::check_input")) {
     check_polynomial_assignment(A, M, lp_polynomial_top_variable(A));
   }
+  assert(A->ctx->K == lp_Z);
 
   // Evaluate the sign and check
   int p_sign = lp_polynomial_sgn(A, M);
   return lp_sign_condition_consistent(sgn_condition, p_sign);
 }
 
+int lp_polynomial_constraint_evaluate_Zp(const lp_polynomial_t* A, lp_sign_condition_t sgn_condition, const lp_assignment_t* m) {
+
+  lp_polynomial_external_clean(A);
+
+  if (trace_is_enabled("polynomial::check_input")) {
+    check_polynomial_assignment(A, m, lp_polynomial_top_variable(A));
+  }
+  assert(A->ctx->K != lp_Z);
+  assert(lp_sign_condition_Zp_valid(sgn_condition));
+  assert(lp_assignment_is_integer(m));
+
+  // Evaluate the sign and check
+  lp_integer_t value;
+  lp_integer_construct(&value);
+  lp_polynomial_evaluate_integer(A, m, &value);
+  int p_sign = !lp_integer_is_zero(A->ctx->K, &value);
+  lp_integer_destruct(&value);
+  return lp_sign_condition_consistent(sgn_condition, p_sign);
+}
+
 int lp_polynomial_root_constraint_evaluate(const lp_polynomial_t* A, size_t root_index, lp_sign_condition_t sgn_condition, const lp_assignment_t* M) {
 
   int eval;
@@ -2183,3 +2394,18 @@ int lp_polynomial_constraint_resolve_fm(
   return ok;
 }
 
+void lp_polynomial_reduce_degree_Zp(lp_polynomial_t *R, const lp_polynomial_t *A) {
+  lp_polynomial_external_clean(A);
+
+  const lp_polynomial_context_t *ctx = A->ctx;
+
+  if (R == A) {
+    coefficient_reduce_Zp(ctx, &R->data);
+  } else {
+    lp_polynomial_t tmp;
+    lp_polynomial_construct_copy(&tmp, A);
+    coefficient_reduce_Zp(ctx, &tmp.data);
+    lp_polynomial_swap(&tmp, R);
+    lp_polynomial_destruct(&tmp);
+  }
+}
