@@ -140,6 +140,9 @@ namespace poly {
   bool is_assigned_over_assignment(const Polynomial& p, const Assignment& a) {
     return lp_polynomial_is_assigned(p.get_internal(), a.get_internal());
   }
+  UPolynomial to_univariate(const Polynomial& p, const Assignment& a) {
+    return UPolynomial(lp_polynomial_to_univariate_m(p.get_internal(), a.get_internal()));
+  }
   int sgn(const Polynomial& p, const Assignment& a) {
     return lp_polynomial_sgn(p.get_internal(), a.get_internal());
   }
@@ -296,6 +299,22 @@ namespace poly {
                         rhs.get_internal());
     return res;
   }
+  std::pair<Polynomial, Polynomial> p_div_rem(const Polynomial& lhs,
+                                              const Polynomial& rhs) {
+    Polynomial d(detail::context(lhs, rhs));
+    Polynomial r(detail::context(lhs, rhs));
+    lp_polynomial_pdivrem(d.get_internal(), r.get_internal(), lhs.get_internal(),
+                         rhs.get_internal());
+    return {d, r};
+  }
+  std::pair<Polynomial, Polynomial> sp_div_rem(const Polynomial& lhs,
+                                               const Polynomial& rhs) {
+    Polynomial d(detail::context(lhs, rhs));
+    Polynomial r(detail::context(lhs, rhs));
+    lp_polynomial_spdivrem(d.get_internal(), r.get_internal(), lhs.get_internal(),
+                          rhs.get_internal());
+    return {d, r};
+  }
   std::pair<Polynomial, Polynomial> div_rem(const Polynomial& lhs,
                                             const Polynomial& rhs) {
     Polynomial d(detail::context(lhs, rhs));
@@ -358,15 +377,31 @@ namespace poly {
 
   std::vector<Polynomial> psc(const Polynomial& p, const Polynomial& q) {
     std::size_t size = std::min(degree(p), degree(q)) + 1;
-    lp_polynomial_t* tmp[size];
+    std::vector<lp_polynomial_t*> tmp;
     for (std::size_t i = 0; i < size; ++i) {
-      tmp[i] = lp_polynomial_new(detail::context(p, q));
+      tmp.push_back(lp_polynomial_new(detail::context(p, q)));
     }
-    lp_polynomial_psc(tmp, p.get_internal(), q.get_internal());
+    lp_polynomial_psc(tmp.data(), p.get_internal(), q.get_internal());
     std::vector<Polynomial> res;
     for (std::size_t i = 0; i < size; ++i) {
       res.emplace_back(tmp[i]);
     }
+    tmp.clear();
+    return res;
+  }
+
+  std::vector<Polynomial> subres(const Polynomial& p, const Polynomial& q) {
+    std::size_t size = std::min(degree(p), degree(q)) + 1;
+    std::vector<lp_polynomial_t*> tmp;
+    for (std::size_t i = 0; i < size; ++i) {
+      tmp.push_back(lp_polynomial_new(detail::context(p, q)));
+    }
+    lp_polynomial_subres(tmp.data(), p.get_internal(), q.get_internal());
+    std::vector<Polynomial> res;
+    for (std::size_t i = 0; i < size; ++i) {
+      res.emplace_back(tmp[i]);
+    }
+    tmp.clear();
     return res;
   }
 
