@@ -31,11 +31,11 @@
 
 #include <stdlib.h>
 #include <assert.h>
-#include <string.h>
 
-#include <upolynomial.h>
 #include "upolynomial/upolynomial.h"
 
+#include <polynomial.h>
+
 size_t lp_upolynomial_degree(const lp_upolynomial_t* p) {
   assert(p);
   assert(p->size > 0);
@@ -43,6 +43,7 @@ size_t lp_upolynomial_degree(const lp_upolynomial_t* p) {
 }
 
 // Construct the polynomial, but don't construct monomials
+static
 lp_upolynomial_t* lp_upolynomial_construct_empty(const lp_int_ring_t* K, size_t size) {
   size_t malloc_size = sizeof(lp_upolynomial_t) + size*sizeof(ulp_monomial_t);
   lp_upolynomial_t* new_p = (lp_upolynomial_t*) malloc(malloc_size);
@@ -250,7 +251,7 @@ const lp_integer_t* lp_upolynomial_const_term(const lp_upolynomial_t* p) {
   if (p->monomials[0].degree == 0) {
     return &p->monomials[0].coefficient;
   } else {
-    return 0;
+    return NULL;
   }
 }
 
@@ -773,7 +774,7 @@ void lp_upolynomial_div_rem_exact(const lp_upolynomial_t* p, const lp_upolynomia
     upolynomial_dense_t rem_buffer;
     upolynomial_dense_t div_buffer;
     lp_upolynomial_div_general(p, q, &div_buffer, &rem_buffer, /** exact */ 1);
-    *div= upolynomial_dense_to_upolynomial(&div_buffer, K);
+    *div = upolynomial_dense_to_upolynomial(&div_buffer, K);
     *rem = upolynomial_dense_to_upolynomial(&rem_buffer, K);
     upolynomial_dense_destruct(&div_buffer);
     upolynomial_dense_destruct(&rem_buffer);
@@ -1090,6 +1091,7 @@ void lp_upolynomial_solve_bezout(const lp_upolynomial_t* p, const lp_upolynomial
 }
 
 lp_upolynomial_factors_t* lp_upolynomial_factor(const lp_upolynomial_t* p) {
+  assert(p);
 
   if (trace_is_enabled("factorization")) {
     tracef("upolynomial_factor("); lp_upolynomial_print(p, trace_out); tracef(")\n");
@@ -1155,6 +1157,15 @@ void lp_upolynomial_sturm_sequence(const lp_upolynomial_t* f, lp_upolynomial_t**
   free(S_dense);
 }
 
+void lp_upolynomial_roots_find_Zp(const lp_upolynomial_t* f, lp_integer_t** roots, size_t* roots_size) {
+  if (trace_is_enabled("roots")) {
+    tracef("upolynomial_roots_find_Zp("); lp_upolynomial_print(f, trace_out); tracef(")\n");
+  }
+  assert(f->K != lp_Z);
+
+  upolynomial_roots_find_Zp(f, roots, roots_size);
+}
+
 lp_upolynomial_t* lp_upolynomial_neg(const lp_upolynomial_t* p) {
   lp_upolynomial_t* neg = lp_upolynomial_construct_copy(p);
   lp_upolynomial_neg_in_place(neg);
@@ -1168,6 +1179,27 @@ void lp_upolynomial_neg_in_place(lp_upolynomial_t* p) {
   }
 }
 
+lp_upolynomial_t* lp_upolynomial_make_monic(const lp_upolynomial_t* p) {
+  lp_upolynomial_t* monic = lp_upolynomial_construct_copy(p);
+  lp_upolynomial_make_monic_in_place(monic);
+  return monic;
+}
+
+void lp_upolynomial_make_monic_in_place(lp_upolynomial_t* p) {
+  if (lp_upolynomial_is_zero(p)) {
+    return;
+  }
+  const lp_int_ring_t* K = p->K;
+
+  const lp_integer_t *c = lp_upolynomial_lead_coeff(p);
+  assert(integer_cmp_int(K, c, 0));
+  for (size_t i = 0; i < p->size; ++i) {
+    assert(integer_divides(K, c, &p->monomials[i].coefficient));
+    integer_div_exact(K, &p->monomials[i].coefficient, &p->monomials[i].coefficient, c);
+  }
+  assert(lp_upolynomial_is_monic(p));
+}
+
 void lp_upolynomial_reverse_in_place(lp_upolynomial_t* p) {
   size_t p_deg;
   ulp_monomial_t tmp, *m_i, *m_j;
@@ -1206,3 +1238,22 @@ lp_upolynomial_t* lp_upolynomial_subst_x_neg(const lp_upolynomial_t* f) {
 
   return neg;
 }
+
+lp_polynomial_t* lp_upolynomial_to_polynomial(const lp_upolynomial_t* p, const lp_polynomial_context_t* ctx, lp_variable_t var) {
+  assert(p);
+  assert(lp_int_ring_equal(p->K, ctx->K));
+
+  lp_polynomial_t *poly = lp_polynomial_new(ctx);
+
+  lp_monomial_t mono;
+  lp_monomial_construct(ctx, &mono);
+  for (size_t i = 0; i < p->size; ++i) {
+    lp_monomial_set_coefficient(ctx, &mono, &p->monomials[i].coefficient);
+    lp_monomial_push(&mono, var, p->monomials[i].degree);
+    lp_polynomial_add_monomial(poly, &mono);
+    lp_monomial_clear(ctx, &mono);
+  }
+  lp_monomial_destruct(&mono);
+
+  return poly;
+}
